package com.zilber.boot.intelligencesite.controller;


import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.zilber.boot.annotation.Anonymous;
import com.zilber.boot.commoncontroller.BaseController;
import com.zilber.boot.intelligencesite.entity.IProductionPlan;
import com.zilber.boot.intelligencesite.entity.IProgress;
import com.zilber.boot.intelligencesite.service.IIProgressService;
import com.zilber.boot.utils.AjaxResult;
import com.zilber.boot.utils.TreeBuild;
import com.zilber.boot.utils.TreeBuild1;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.*;

/**
 * <p>
 * 进度跟踪表 前端控制器
 * </p>
 *
 * @author lsc
 * @since 2025-05-06
 */
@RestController
@RequestMapping("/iprogress")
@Api(tags = "进度跟踪")
public class IProgressController extends BaseController {

    @Resource
    private IIProgressService iiProgressService;


    @GetMapping("/list")
    @ApiOperation(value="分页查询",notes="分页查询")
    @ResponseBody
    public AjaxResult queryList(IProgress iProgress, @RequestParam(defaultValue = "1") Integer pageNo,
                                @RequestParam(defaultValue = "10") Integer pageSize) {
        List<IProgress> iProgresses = iiProgressService.queryListByContion(iProgress);
        TreeBuild1 treeBuild = new TreeBuild1(iProgresses);
        // 原查询结果转换树形结构
        List<IProgress> list = treeBuild.buildTree();
        // 原查询结果转换树形结构
        PageHelper.startPage(pageNo, pageSize);
        PageInfo<IProgress> page = new PageInfo<>(list);
        return AjaxResult.success(page);
    }

    /***
     * 查询单个
     */
    @GetMapping("/getById")
    @ApiOperation("查询单个")
    public AjaxResult getById(Integer id) {
        return AjaxResult.success(iiProgressService.getById(id));
    }


    @PostMapping("/add")
    @ApiOperation("增加")

    public AjaxResult add(@RequestBody IProgress iProgress) {
        iProgress.setCreateTime(new Date());
        iProgress.setCreator(getUsername());
        return iiProgressService.add(iProgress);
    }


    @PutMapping("/update")
    @ApiOperation("修改")
    public AjaxResult update(@RequestBody @Validated IProgress iProgress) {
        iiProgressService.updateById(iProgress);
        return AjaxResult.success();
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除")
    public AjaxResult delete(Integer id) {
        return AjaxResult.success(iiProgressService.removeById(id));
    }

}
