package com.zilber.boot.intelligencesite.service.impl;

import com.zilber.boot.intelligencesite.entity.*;
import com.zilber.boot.intelligencesite.mapper.*;
import com.zilber.boot.intelligencesite.service.IIProductionPlanService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zilber.boot.utils.AjaxResult;
import com.zilber.boot.utils.DateUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.*;

/**
 * <p>
 * 生产计划表 服务实现类
 * </p>
 *
 * @author lsc
 * @since 2025-05-06
 */
@Service
public class IProductionPlanServiceImpl extends ServiceImpl<IProductionPlanMapper, IProductionPlan> implements IIProductionPlanService {

    @Resource
    private IProductionPlanMapper iProductionPlanMapper;
    @Resource
    private IProgressMapper iProgressMapper;
    @Resource
    private IManpowerMapper iManpowerMapper;
    @Resource
    private IWarnMapper iWarnMapper;
    @Resource
    private IProgressNewMapper iProgressNewMapper;


    @Override
    public List<IProductionPlan> queryList(IProductionPlan iProductionPlan) {
        return iProductionPlanMapper.queryList(iProductionPlan);
    }

    @Override
    public List<Map<String, Object>> statistics() {
        List<String> recentWeekDates = getRecentWeekDates();
        List<Map<String, Object>> res = new ArrayList<>();
        List<IProductionPlan> iProductionPlans = iProductionPlanMapper.statistics();
        if (iProductionPlans.size() > 0) {
            for (IProductionPlan iProductionPlan : iProductionPlans) {
                for (int i = 0; i < recentWeekDates.size(); i++) {
                   IProductionPlan iP = iProductionPlanMapper.statisticsByDateAndName(iProductionPlan.getId(),recentWeekDates.get(i));
                    Map map = new HashMap();
                   if(ObjectUtils.isEmpty(iP)){
                       map.put("name", iProductionPlan.getPlanName());
                       BigDecimal daypercent = new BigDecimal(1).divide(new BigDecimal(iProductionPlan.getDuration()), 2, BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal(100));

                       map.put("plan", daypercent);
                       map.put("date", recentWeekDates.get(i));
                       List<IProgressNew> progresseslist = iProgressNewMapper.queryProcessSj(iProductionPlan.getId(),recentWeekDates.get(i));
                       long total = progresseslist.stream().mapToLong(IProgressNew::getDayProgress).sum();
                       map.put("sj", total);
                       res.add(map);
                   }else{
                       //获取每个计划时间差（结束时间-开始时间）
                       //long datePoorByDay = DateUtils.getDatePoorByDay(iProductionPlan.getEndTime(), iProductionPlan.getStartTime());
                       //获取每个计划每天应完成的百分比
                       BigDecimal currentpercent = new BigDecimal(1).divide(new BigDecimal(iProductionPlan.getDuration()), 2, BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal(100));
                       //获取当前日期，减去开始日期，乘以每个计划每天应完成的百分比，算出截止今天应该完成的百分比
                  /*     BigDecimal currentpercent = BigDecimal.valueOf(DateUtils.getDatePoorByDay(new Date(), iProductionPlan.getStartTime())).multiply(daypercent);
                       if (DateUtils.getDatePoorByDay(new Date(), iProductionPlan.getEndTime()) > 0) {
                           currentpercent = BigDecimal.valueOf(100);
                       }*/

                       map.put("name", iProductionPlan.getPlanName());
                       map.put("plan", currentpercent);
                       map.put("date", recentWeekDates.get(i));
                       //查询计划实际填报情况
                       List<IProgressNew> progresseslist = iProgressNewMapper.queryProcessSj(iProductionPlan.getId(),recentWeekDates.get(i));
                       long total = progresseslist.stream().mapToLong(IProgressNew::getDayProgress).sum();
                 /*      List<IProgress> progresseslist = iProgressMapper.queryList(new IProgress().setPlanId(iProductionPlan.getId()));
                       long total = progresseslist.stream().mapToLong(IProgress::getDayProgress).sum();*/
                       map.put("sj", total);
                       res.add(map);
                   }
                }
            }
        }
        return res;
    }

    @Override
    public List<IProductionPlan> querywarn(Integer day) {
        List<IProductionPlan> iProductionPlans = iProductionPlanMapper.querywarn();
        List<IProductionPlan> newiProductionPlans = new ArrayList<>();
        for (IProductionPlan iProductionPlan : iProductionPlans) {
            if (DateUtils.getDatePoorByDay(new Date(), iProductionPlan.getEndTime()) > day) {
                newiProductionPlans.add(iProductionPlan);
                IWarn i = iWarnMapper.selectByPlanId(iProductionPlan.getId());
                if (ObjectUtils.isEmpty(i)) {
                    IWarn warn = new IWarn();
                    warn.setPlanId(iProductionPlan.getId());
                    warn.setPlanName(iProductionPlan.getPlanName());
                    warn.setCreateTime(new Date());
                    iWarnMapper.insert(warn);
                }
            }
        }
        return newiProductionPlans;
    }

    @Override
    public Map<String, Object> dataOverview() {
        int total = iProductionPlanMapper.queryTotal();//总工期
        int sg = iManpowerMapper.querySg();//施工人员
        int gl = iProductionPlanMapper.queryGl();//管理人员
        List<IProgressNew> ysg = iProgressNewMapper.queryYsg();  //已施工
        //int gcjd=iProductionPlanMapper.queryGcjd();//工程进度
        Map map = new HashMap();
        map.put("list", total);//总工期
        map.put("sg", sg);//施工人员
        map.put("gl", gl);//管理人员
        map.put("ysg", ysg.size());// //已施工
        //查询一级计划
        double day = 0;
        int totalday = 0;
        List<IProductionPlan> plan = iProductionPlanMapper.bigscreenProgress();
        for (int i = 0; i < plan.size(); i++) {
            if (null != plan.get(i).getAccumulativeProgress()) {
                day += plan.get(i).getDuration() * plan.get(i).getAccumulativeProgress();
            } else {
                day += 0;

            }
            totalday += plan.get(i).getDuration();
        }
        map.put("gcjd", day / totalday);// //已施工

        return AjaxResult.success(map);
    }

    @Override
    public List<IProductionPlan> bigscreenProgress() {
        return iProductionPlanMapper.bigscreenProgress();
    }



    //获取最近一周时间
    public static List<String> getRecentWeekDates() {
        List<String> dates = new ArrayList<>();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        LocalDate today = LocalDate.now();

        for (int i = 0; i < 7; i++) {
            LocalDate date = today.minusDays(i);
            dates.add(date.format(formatter));
        }

        return dates;
    }

    @Override
    public List<IProductionPlan> queryListNew(IProductionPlan iProductionPlan,Date date) {
        List<IProductionPlan> iProductionPlans = iProductionPlanMapper.queryList(iProductionPlan);
        for(int i=0;i<iProductionPlans.size();i++){
            IProgress ip=iProgressMapper.queryByIdAndDate(iProductionPlans.get(i).getId(),date) ;
            if(!ObjectUtils.isEmpty(ip)){
                iProductionPlans.get(i).setDayProgress(ip.getDayProgress());
            }
            List<IProgress> ips=iProgressMapper.queryById(iProductionPlans.get(i).getId()) ;
            if(!ObjectUtils.isEmpty(ips)){
                long total = ips.stream().mapToLong(IProgress::getDayProgress).sum();
                iProductionPlans.get(i).setAccumulativeProgress(Integer.valueOf(String.valueOf(total)));
            }

        }
        return iProductionPlans;
    }

}
