package com.zilber.boot.module.vehicle.service.impl;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.ArrayUtils;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.zilber.boot.exception.ServiceException;
import com.zilber.boot.module.vehicle.constant.Constant;
import com.zilber.boot.module.vehicle.dto.pr.DevicePR;
import com.zilber.boot.module.vehicle.enums.Method;
import com.zilber.boot.module.vehicle.properties.TourRunProperties;
import com.zilber.boot.module.vehicle.service.TourRunService;
import com.zilber.boot.module.vehicle.util.SignUtils;
import com.zilber.boot.utils.StringUtils;
import com.zilber.boot.utils.page.PageUtils;
import org.apache.commons.codec.digest.DigestUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.net.HttpURLConnection;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Predicate;
import java.util.stream.Collectors;

import static com.zilber.boot.module.vehicle.util.http.HttpUtils.sendGet;
import static com.zilber.boot.module.vehicle.util.http.HttpUtils.sendPost;

/**
 * @Author LJX
 * @TIME 2025-05-06 11:56
 * @PROJECT intelligence-site
 * created by Intellij IDEA
 * Description
 */
@Service
public class TourRunServiceImpl
        implements TourRunService {

    // 申请来的appKey和appSecret
//    @Value("${tourRun.tourRunPassword}")
//    private String appKey;
//
//    @Value("${tourRun.tourRunPassword}")
//    private String appSecret;
//
//    @Value("${tourRun.tourRunPassword}")
//    private String tourRunAccount;
//
//    @Value("${tourRun.tourRunPassword}")
//    private String tourRunPassword;
//
//    @Value("${tourRun.openapiUrl}")
//    private String openapiUrl;
//
//    @Value("${tourRun.h5Url}")
//    private String h5Url;

    @Autowired
    private TourRunProperties properties;

    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Override
    public PageUtils list(DevicePR pr) {
//        String token = token();
//
//        Map<String, String> paramMap = paramMap(Method.LIST.getCode());
//
//        // 私有参数
//        paramMap.put("access_token", token);
//        paramMap.put("target", properties.getTourRunAccount());
//        // 计算签名
//        String sign = "";
//        try {
//            sign = SignUtils.signTopRequest(paramMap, properties.getAppSecret(), Constant.MD5);
//            paramMap.put("sign", sign);
//        } catch (IOException e) {
//            e.printStackTrace();
//        }
//        //调用com/zilber/boot/module/vehicle/util/http/HttpUtils方法
//        JSONObject jsonObject = sendPost(properties.getOpenapiUrl(), headerMap(), paramMap);
//        //初始化列表
////        List<JSONObject> resultList = null;
////        if ( jsonObject != null) {
////            if ( !jsonObject.get("code").toString().equals("0"))
////            {
////                throw new ServiceException(jsonObject.get("message").toString());
////            }
////            resultList = jsonObject.getList("result", JSONObject.class);
////        }
////        else {
////            throw new ServiceException("列表获取失败");
////        }
//        List<JSONObject> resultList = handleResult(jsonObject, JSONObject.class, "获取列表失败");
        List<JSONObject> resultList = resultList();
        if (CollectionUtils.isNotEmpty(resultList))
        {
            List<JSONObject> handledList = resultList.stream()
                    .filter(jo -> jo.get("mcTypeUseScope").toString().equals(pr.getScope()))
                    .sorted((o1, o2) -> StringUtils.compare(o1.get("imei").toString(), o2.get("imei").toString()))
                    .collect(Collectors.toList());
            return new PageUtils(
                    //将处理后的列表手动分页
                    handledList.stream()
                            .skip((long) (pr.getPageNo() - 1) *pr.getPageSize())
                            .limit(pr.getPageSize())
                            .collect(Collectors.toList()),
                    handledList.size(),
                    pr.getPageSize(),
                    pr.getPageNo());
        }
        return new PageUtils(new ArrayList<>(), 0, pr.getPageSize(), pr.getPageNo());
    }

    @Override
    public Map<String, Integer> stat() {
        Map<String, Integer> resultMap = new HashMap<>(Constant.SCOPES.size());
        List<JSONObject> resultList = resultList();
        if (CollectionUtils.isNotEmpty(resultList)) {
            Map<String, List<JSONObject>> scopeMap
                    = resultList.stream()
                    .collect(Collectors.groupingBy(jo -> jo.get("mcTypeUseScope").toString()));
            for ( String scope: Constant.SCOPES) {
                List<JSONObject> jsonObjects = scopeMap.get(scope);
                if ( CollectionUtils.isNotEmpty(jsonObjects)) {
                    resultMap.put(scope, jsonObjects.size());
                }
                else {
                    resultMap.put(scope, 0);
                }
            }
            return resultMap;
        }
        for ( String scope: Constant.SCOPES) {
            resultMap.put(scope, 0);
        }
        return resultMap;
    }

    //https://openh5.aichezaixian.com/?token=XXX&imei=AAA&appKey=BBB&activeTab=CCC
    @Override
    public String h5(String tabType, String imei) {
        if ( StringUtils.isEmpty(tabType)) {
            throw new ServiceException("tab栏位不能为空");
        }
        if ( StringUtils.isEmpty(imei)){
            throw new ServiceException("imei号不能为空");
        }
        String suffix = "/?token={}&imei={}&appKey={}&activeTab={}";
        String token = token();

        String format = StringUtils.format(suffix, token, imei, properties.getAppKey(), tabType);
        return sendGet(properties.getH5Url() + format);
    }

    @Override
    public String token() {
        String token = redisTemplate.opsForValue().get(Constant.KEY);
        if (StringUtils.isEmpty(token)) {

            Map<String, String> paramMap = paramMap(Method.TOKEN.getCode());

            // 私有参数
            paramMap.put("user_id", properties.getTourRunAccount());
            paramMap.put("user_pwd_md5", DigestUtils.md5Hex(properties.getTourRunPassword()));
            paramMap.put("expires_in", "120");

            // 计算签名
            String sign = "";
            try {
                sign = SignUtils.signTopRequest(paramMap, properties.getAppSecret(), Constant.MD5);
                paramMap.put("sign", sign);
            } catch (IOException e) {
                e.printStackTrace();
            }
            //调用com/zilber/boot/module/vehicle/util/http/HttpUtils方法
            JSONObject jsonObject = sendPost(properties.getOpenapiUrl(), headerMap(), paramMap);
//            if ( jsonObject != null) {
//                if ( !jsonObject.get("code").toString().equals("0"))
//                {
//                    throw new ServiceException(jsonObject.get("message").toString());
//                }
//                token = ((JSONObject) jsonObject.get("result")).get("accessToken").toString();
//            }
//            else {
//                throw new ServiceException("无法获取token");
//            }
            token = handleResult(jsonObject, "无法获取token");
            redisTemplate.opsForValue().set(Constant.KEY, token, Constant.TIMEOUT, TimeUnit.SECONDS);
        }
        return token;
    }

    public static String getCurrentDate() {
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        return formatter.format(new Date());
    }

    private List<JSONObject> resultList() {
        String token = token();

        Map<String, String> paramMap = paramMap(Method.LIST.getCode());

        // 私有参数
        paramMap.put("access_token", token);
        paramMap.put("target", properties.getTourRunAccount());
        // 计算签名
        String sign = "";
        try {
            sign = SignUtils.signTopRequest(paramMap, properties.getAppSecret(), Constant.MD5);
            paramMap.put("sign", sign);
        } catch (IOException e) {
            e.printStackTrace();
        }
        //调用com/zilber/boot/module/vehicle/util/http/HttpUtils方法
        JSONObject jsonObject = sendPost(properties.getOpenapiUrl(), headerMap(), paramMap);
        //初始化列表
//        List<JSONObject> resultList = null;
//        if ( jsonObject != null) {
//            if ( !jsonObject.get("code").toString().equals("0"))
//            {
//                throw new ServiceException(jsonObject.get("message").toString());
//            }
//            resultList = jsonObject.getList("result", JSONObject.class);
//        }
//        else {
//            throw new ServiceException("列表获取失败");
//        }
        return handleResult(jsonObject, JSONObject.class, "获取列表失败");
    }

    /**
     * 构造并初始化部分参数的请求头map
     * @return map
     */
    private Map<String, String> headerMap(){
        Map<String, String> headerMap = new HashMap<>();
        headerMap.put("Content-Type", "application/x-www-form-urlencoded");
        return headerMap;
    }

    /**
     * 构造并初始化部分参数的请求参数map
     * @return map
     */
    private Map<String, String> paramMap(String method){
        Map<String, String> paramMap = new HashMap<>();
        // 公共参数
        paramMap.put("app_key", properties.getAppKey());
        paramMap.put("v", "1.0");
        paramMap.put("timestamp", getCurrentDate());
        paramMap.put("sign_method", Constant.MD5);
        paramMap.put("format", "json");
        paramMap.put("method", method);
        return paramMap;
    }

    /**
     * 处理接收到的返回结果（列表）
     * @param result 返回结果
     * @param clazz 数据类
     * @param errorMessage 自定义错误信息（result为空时的异常信息）
     * @return
     * @param <T>
     */
    private <T> List<T> handleResult(JSONObject result, Class<T> clazz, String errorMessage) {
        List<T> resultList = null;
        if ( result != null) {
            if ( !result.get(Constant.CODE).toString().equals(Constant.SUCCESS_CODE)) {
                throw new ServiceException(result.get(Constant.message).toString());
            }
            resultList = result.getList(Constant.RESULT, clazz);
            return resultList;
        }
        else {
            throw new ServiceException(errorMessage);
        }
    }

    /**
     * 处理接收到的返回结果（字符串）
     * @param result 结果
     * @param errorMessage 自定义错误信息（result为空时的异常信息）
     * @return
     */
    private String handleResult(JSONObject result, String errorMessage) {
        if ( result != null) {
            if ( !result.get(Constant.CODE).toString().equals(Constant.SUCCESS_CODE)) {
                throw new ServiceException(result.get(Constant.message).toString());
            }
            return result.getJSONObject("result").get("accessToken").toString();
        }
        else {
            throw new ServiceException(errorMessage);
        }
    }


}
